/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ow2.jasmine.rules.AbstractDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * A OSGi service implementation to manage a set of working memories
 *
 * @author Julien Nicoulaud
 * @see {@link AbstractDroolsWorkingMemoryPool}
 */
public class OSGiDroolsWorkingMemoryPool extends AbstractDroolsWorkingMemoryPool
implements IOSGiDroolsWorkingMemoryPool {

    // -------------------------------------------------------------------------
    // Variables
    // -------------------------------------------------------------------------

    /**
     * Logger
     */
    private static Logger logger =
        Logger.getLogger(OSGiDroolsWorkingMemoryPool.class.getName());


    private final BundleContext context;


    // -------------------------------------------------------------------------
    // Constructors
    // -------------------------------------------------------------------------

    /**
     * Create a pool, with the default class loader to use to construct
     * new managed working memories.
     */
    public OSGiDroolsWorkingMemoryPool(final BundleContext context) {
        this.context = context;
    }


    // -------------------------------------------------------------------------
    // Implementation of IDroolsWorkingMemoryPool
    // -------------------------------------------------------------------------

    /**
     * Get a WorkingMemory in the set, create it if does
     * not exist.
     * The potentially created WorkingMemory uses a default
     * ClassLoader.
     *
     * @param workingMemoryName the name of the new WorkingMemory
     * @throws Exception if a working memory could not be created.
     */
    public synchronized IDroolsWorkingMemory getOrCreate(final String workingMemoryName) throws Exception{

        IDroolsWorkingMemory result = this.get(workingMemoryName);
        if (result == null) {
            logger.log(Level.INFO,"There is no WorkingMemory named \""
                    + workingMemoryName + "\" in the pool. Creating it...");

            // Create a new working memory with the default class loader
            result =
                new OSGiDroolsWorkingMemory(context,
                        new HashMap<ServiceReference, IPackageAdder>());

            // Add it to the pool
            this.add(workingMemoryName, result);

        }
        return result;
    }

}
