/**
 * JASMINe
 * Copyright (C) 20010 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.rules.osgi.impl;

import org.osgi.framework.BundleContext;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemoryPool;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.testng.Assert.assertFalse;

/**
 * A {@code OSGiDroolsWorkingMemoryTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class OSGiDroolsWorkingMemoryPoolTestCase {

    @Test
    public void testMultiThreadedGetOrCreate() throws Exception {

        // Mock the BundleContext
        BundleContext bundleContext = mock(BundleContext.class);

        // Create the pool
        OSGiDroolsWorkingMemoryPool pool = new OSGiDroolsWorkingMemoryPool(bundleContext);

        // Init the Threads
        TestingThread t1 = new TestingThread("one", 10);
        TestingThread t2 = new TestingThread("two", 10);
        t1.pool = pool;
        t2.pool = pool;

        // Start them
        t1.start();
        t2.start();

        // Wait for them to stop
        t1.join();
        t2.join();

        // Ensure that there were no errors
        assertFalse(t1.error);
        assertFalse(t2.error);


    }

    private class TestingThread extends Thread {
        IDroolsWorkingMemoryPool pool;
        boolean error = false;
        private int times;

        private TestingThread(String name, int times) {
            super(name);
            this.times = times;
        }

        @Override
        public void run() {
            int i = 0;
            while (i++ < times) {
                try {
                    pool.getOrCreate("multithreaded");
                } catch (Exception e) {
                    e.printStackTrace();
                    error = true;
                }
            }
        }
    }


}
