/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.rules.manager.persistence;

import java.util.List;

import org.ow2.jasmine.rules.manager.model.IRulePackage;

/**
 * The remote object interface that exposes the rules persistence API
 * 
 * @author HAN Tianyi, Julien Nicoulaud
 */
public interface RulePackageSLBRemote {
	
    /**
     * Implementation of inherited method: creates the Rule entity bean
     * corresponding to given data and saves the rule on the database.
     *
     * @param r
     *            Rule to save.
     */
    void saveRulePackage(final IRulePackage r) throws Exception;
    
    /**
     * 
     * @param r
     *            Rule to delete.
     * @throws Exception
     */
    void deleteRulePackage(IRulePackage r) throws Exception;

    /**
     * Implementation of the inherited method: looks for the beans corresponding,
     * converts the search into an array of RulePackage objects and return them back.
     *
     * @see RulePackage[] getRules(Integer, String, String, String, String,
     *      String, String, String, Integer);
     *
     * @param id
     *            the ID of rule
     * @param systemName
     *            The unique system name of the rule package
     * @param displayName
     *            The displayed name of the rule package
     * @param description
     *            The description of the rule package
     * @param state
     *            The state of the rule package
     * @param drlSource
     *            The DRL source of the rule package
     * @param ruleEngineName
     *            The engine name of the rule
     * @param orderBy
     *            Order using column.
     * @param limit
     *            Maximal number of results to get.
     *
     * @return Rules corresponding to the given search, null if none found.
     */
    IRulePackage[] getRulePackages(final Integer id,
                                   final String systemName,
                                   final String displayName,
                                   final String ruleEngineInstanceName,
                                   final String description,
                                   final String state,
                                   final String orderBy,
                                   final Integer limit) throws Exception;
    
    /**
     * Get the number of rules stored
     *
     * @return size
     *              the number of rules
     */
    int countRulePackages();
    
    /**
     * Get the number of rules stored for a specific rule engine instance
     *
     * @return size
     *              the number of rules
     */
    int countRulePackages(final String ruleEngineInstanceName);
        
    /**
     * Get the names of the target rule engine instances for which a manager
     * should be created. This the rule engine instances that have rules marked
     * as activated in the database.
     * 
     * @return List<String>
     *             return the table as a result
     */
    List<String> getRuleEngineInstancesToManage();
    
    /**
     * Check a rule package name availability
     */
    Boolean checkNameAvailability(String systemName,
    							  String ruleEngineInstanceName) throws Exception;

}
