/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.service;

import java.util.HashMap;
import java.util.Map;

import org.osgi.framework.BundleContext;

/**
 * An OSGi service class aimed at providing managers for the rule engine instances
 * to clients.
 * 
 * @author Julien Nicoulaud
 * @see {@link org.ow2.jasmine.rules.manager.service.IDroolsManager}
 */
public class DroolsManagerFactory implements IDroolsManagerFactory {
	
	// =========================================================================
	// Variables
	// =========================================================================
	
	/**
	 * The bundle context
	 */
	private BundleContext bundleContext;
	
	/**
	 * The Drools Managers
	 */
	public Map<String,IDroolsManager> droolsManagers =
		new HashMap<String,IDroolsManager>();
	
	
	// =========================================================================
	// Constructors
	// =========================================================================
	
	/**
	 * Build a new DroolsManagerFactory
	 * @param bundleContext the bundle context
	 */
	public DroolsManagerFactory(BundleContext bundleContext) {
		this.bundleContext = bundleContext;
	}
	
	
	// =========================================================================
	// Implementation of IDroolsManagerFactory
	// =========================================================================
	
	/**
	 * Get the manager for the given instance of the rule engine. If the
	 * manager and the rule engine instance do not exist, they are automatically
	 * created at the manager instanciation.
	 * 
	 * @param ruleEngineName the name of the rule engine instance
	 * @return the manager
	 * @throws Exception if no valid manager could be returned
	 */
	public IDroolsManager getManager(String ruleEngineName) throws Exception {
		
		if(!droolsManagers.containsKey(ruleEngineName)){

			// Create a manager for our working memory
			droolsManagers.put(ruleEngineName,new DroolsManager(ruleEngineName,bundleContext));
		}
		return droolsManagers.get(ruleEngineName);
	}
}
